# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'nanogen.ui'
#
# Created: Sun Mar 16 13:07:48 2014
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_NanoGen(object):
    def setupUi(self, NanoGen):
        NanoGen.setObjectName(_fromUtf8("NanoGen"))
        NanoGen.resize(532, 729)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Lucida Sans Unicode"))
        font.setPointSize(14)
        NanoGen.setFont(font)
        self.centralwidget = QtGui.QWidget(NanoGen)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.verticalLayout_12 = QtGui.QVBoxLayout(self.centralwidget)
        self.verticalLayout_12.setObjectName(_fromUtf8("verticalLayout_12"))
        self.nanogen_tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.nanogen_tabWidget.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.nanogen_tabWidget.setFont(font)
        self.nanogen_tabWidget.setObjectName(_fromUtf8("nanogen_tabWidget"))
        self.nanotube_generator_tabWidget = QtGui.QWidget()
        self.nanotube_generator_tabWidget.setObjectName(_fromUtf8("nanotube_generator_tabWidget"))
        self.verticalLayout_10 = QtGui.QVBoxLayout(self.nanotube_generator_tabWidget)
        self.verticalLayout_10.setObjectName(_fromUtf8("verticalLayout_10"))
        self.horizontalLayout_38 = QtGui.QHBoxLayout()
        self.horizontalLayout_38.setObjectName(_fromUtf8("horizontalLayout_38"))
        self.verticalLayout_6 = QtGui.QVBoxLayout()
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.nanotube_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.nanotube_generator_radioButton.setFont(font)
        self.nanotube_generator_radioButton.setChecked(True)
        self.nanotube_generator_radioButton.setObjectName(_fromUtf8("nanotube_generator_radioButton"))
        self.nanotube_generator_buttonGroup = QtGui.QButtonGroup(NanoGen)
        self.nanotube_generator_buttonGroup.setObjectName(_fromUtf8("nanotube_generator_buttonGroup"))
        self.nanotube_generator_buttonGroup.addButton(self.nanotube_generator_radioButton)
        self.verticalLayout_2.addWidget(self.nanotube_generator_radioButton)
        self.capped_nanotube_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.capped_nanotube_generator_radioButton.setFont(font)
        self.capped_nanotube_generator_radioButton.setCheckable(False)
        self.capped_nanotube_generator_radioButton.setObjectName(_fromUtf8("capped_nanotube_generator_radioButton"))
        self.nanotube_generator_buttonGroup.addButton(self.capped_nanotube_generator_radioButton)
        self.verticalLayout_2.addWidget(self.capped_nanotube_generator_radioButton)
        self.mwnt_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.mwnt_generator_radioButton.setFont(font)
        self.mwnt_generator_radioButton.setObjectName(_fromUtf8("mwnt_generator_radioButton"))
        self.nanotube_generator_buttonGroup.addButton(self.mwnt_generator_radioButton)
        self.verticalLayout_2.addWidget(self.mwnt_generator_radioButton)
        self.unrolled_nanotube_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.unrolled_nanotube_generator_radioButton.setFont(font)
        self.unrolled_nanotube_generator_radioButton.setObjectName(_fromUtf8("unrolled_nanotube_generator_radioButton"))
        self.nanotube_generator_buttonGroup.addButton(self.unrolled_nanotube_generator_radioButton)
        self.verticalLayout_2.addWidget(self.unrolled_nanotube_generator_radioButton)
        self.bundle_generator_checkBox = QtGui.QCheckBox(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.bundle_generator_checkBox.setFont(font)
        self.bundle_generator_checkBox.setCheckable(True)
        self.bundle_generator_checkBox.setObjectName(_fromUtf8("bundle_generator_checkBox"))
        self.verticalLayout_2.addWidget(self.bundle_generator_checkBox)
        self.verticalLayout_6.addLayout(self.verticalLayout_2)
        self.line = QtGui.QFrame(self.nanotube_generator_tabWidget)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.verticalLayout_6.addWidget(self.line)
        self.horizontalLayout_13 = QtGui.QHBoxLayout()
        self.horizontalLayout_13.setObjectName(_fromUtf8("horizontalLayout_13"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label_7 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.label_7.setFont(font)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.horizontalLayout_2.addWidget(self.label_7)
        self.n_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        self.n_spinBox.setMinimumSize(QtCore.QSize(90, 0))
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.n_spinBox.setFont(font)
        self.n_spinBox.setMaximum(100)
        self.n_spinBox.setProperty("value", 10)
        self.n_spinBox.setObjectName(_fromUtf8("n_spinBox"))
        self.horizontalLayout_2.addWidget(self.n_spinBox)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label_14 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.label_14.setFont(font)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.horizontalLayout.addWidget(self.label_14)
        self.m_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        self.m_spinBox.setMinimumSize(QtCore.QSize(90, 0))
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.m_spinBox.setFont(font)
        self.m_spinBox.setMaximum(100)
        self.m_spinBox.setProperty("value", 10)
        self.m_spinBox.setObjectName(_fromUtf8("m_spinBox"))
        self.horizontalLayout.addWidget(self.m_spinBox)
        self.horizontalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_13.addLayout(self.horizontalLayout_3)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem)
        self.verticalLayout_6.addLayout(self.horizontalLayout_13)
        self.line_2 = QtGui.QFrame(self.nanotube_generator_tabWidget)
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName(_fromUtf8("line_2"))
        self.verticalLayout_6.addWidget(self.line_2)
        self.horizontalLayout_31 = QtGui.QHBoxLayout()
        self.horizontalLayout_31.setObjectName(_fromUtf8("horizontalLayout_31"))
        self.horizontalLayout_30 = QtGui.QHBoxLayout()
        self.horizontalLayout_30.setObjectName(_fromUtf8("horizontalLayout_30"))
        self.horizontalLayout_29 = QtGui.QHBoxLayout()
        self.horizontalLayout_29.setObjectName(_fromUtf8("horizontalLayout_29"))
        self.label_17 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_17.setFont(font)
        self.label_17.setObjectName(_fromUtf8("label_17"))
        self.horizontalLayout_29.addWidget(self.label_17)
        self.nanotube_element1_comboBox = QtGui.QComboBox(self.nanotube_generator_tabWidget)
        self.nanotube_element1_comboBox.setObjectName(_fromUtf8("nanotube_element1_comboBox"))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_29.addWidget(self.nanotube_element1_comboBox)
        self.horizontalLayout_30.addLayout(self.horizontalLayout_29)
        self.horizontalLayout_28 = QtGui.QHBoxLayout()
        self.horizontalLayout_28.setObjectName(_fromUtf8("horizontalLayout_28"))
        self.label_18 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_18.setFont(font)
        self.label_18.setObjectName(_fromUtf8("label_18"))
        self.horizontalLayout_28.addWidget(self.label_18)
        self.nanotube_element2_comboBox = QtGui.QComboBox(self.nanotube_generator_tabWidget)
        self.nanotube_element2_comboBox.setObjectName(_fromUtf8("nanotube_element2_comboBox"))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_28.addWidget(self.nanotube_element2_comboBox)
        self.horizontalLayout_30.addLayout(self.horizontalLayout_28)
        self.horizontalLayout_31.addLayout(self.horizontalLayout_30)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_31.addItem(spacerItem1)
        self.verticalLayout_6.addLayout(self.horizontalLayout_31)
        self.horizontalLayout_24 = QtGui.QHBoxLayout()
        self.horizontalLayout_24.setObjectName(_fromUtf8("horizontalLayout_24"))
        self.horizontalLayout_14 = QtGui.QHBoxLayout()
        self.horizontalLayout_14.setObjectName(_fromUtf8("horizontalLayout_14"))
        self.nanotube_e1e2_bond_label = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nanotube_e1e2_bond_label.setFont(font)
        self.nanotube_e1e2_bond_label.setObjectName(_fromUtf8("nanotube_e1e2_bond_label"))
        self.horizontalLayout_14.addWidget(self.nanotube_e1e2_bond_label)
        self.nanotube_bond_doubleSpinBox = QtGui.QDoubleSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nanotube_bond_doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.nanotube_bond_doubleSpinBox.setSizePolicy(sizePolicy)
        self.nanotube_bond_doubleSpinBox.setMinimumSize(QtCore.QSize(150, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nanotube_bond_doubleSpinBox.setFont(font)
        self.nanotube_bond_doubleSpinBox.setInputMethodHints(QtCore.Qt.ImhFormattedNumbersOnly)
        self.nanotube_bond_doubleSpinBox.setReadOnly(False)
        self.nanotube_bond_doubleSpinBox.setDecimals(3)
        self.nanotube_bond_doubleSpinBox.setMinimum(0.5)
        self.nanotube_bond_doubleSpinBox.setMaximum(10.0)
        self.nanotube_bond_doubleSpinBox.setSingleStep(0.01)
        self.nanotube_bond_doubleSpinBox.setProperty("value", 1.42)
        self.nanotube_bond_doubleSpinBox.setObjectName(_fromUtf8("nanotube_bond_doubleSpinBox"))
        self.horizontalLayout_14.addWidget(self.nanotube_bond_doubleSpinBox)
        self.horizontalLayout_24.addLayout(self.horizontalLayout_14)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_24.addItem(spacerItem2)
        self.verticalLayout_6.addLayout(self.horizontalLayout_24)
        self.line_8 = QtGui.QFrame(self.nanotube_generator_tabWidget)
        self.line_8.setFrameShape(QtGui.QFrame.HLine)
        self.line_8.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_8.setObjectName(_fromUtf8("line_8"))
        self.verticalLayout_6.addWidget(self.line_8)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.horizontalLayout_11 = QtGui.QHBoxLayout()
        self.horizontalLayout_11.setObjectName(_fromUtf8("horizontalLayout_11"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.label = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout_4.addWidget(self.label)
        self.nx_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nx_spinBox.sizePolicy().hasHeightForWidth())
        self.nx_spinBox.setSizePolicy(sizePolicy)
        self.nx_spinBox.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nx_spinBox.setFont(font)
        self.nx_spinBox.setMinimum(1)
        self.nx_spinBox.setObjectName(_fromUtf8("nx_spinBox"))
        self.horizontalLayout_4.addWidget(self.nx_spinBox)
        self.horizontalLayout_11.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_7 = QtGui.QHBoxLayout()
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.label_2 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_2.setFont(font)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout_7.addWidget(self.label_2)
        self.Lx_lineEdit = QtGui.QLineEdit(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Lx_lineEdit.sizePolicy().hasHeightForWidth())
        self.Lx_lineEdit.setSizePolicy(sizePolicy)
        self.Lx_lineEdit.setMinimumSize(QtCore.QSize(175, 0))
        self.Lx_lineEdit.setMaximumSize(QtCore.QSize(200, 16777215))
        self.Lx_lineEdit.setReadOnly(True)
        self.Lx_lineEdit.setObjectName(_fromUtf8("Lx_lineEdit"))
        self.horizontalLayout_7.addWidget(self.Lx_lineEdit)
        self.horizontalLayout_11.addLayout(self.horizontalLayout_7)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_10 = QtGui.QHBoxLayout()
        self.horizontalLayout_10.setObjectName(_fromUtf8("horizontalLayout_10"))
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.label_3 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_3.setFont(font)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout_5.addWidget(self.label_3)
        self.ny_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ny_spinBox.sizePolicy().hasHeightForWidth())
        self.ny_spinBox.setSizePolicy(sizePolicy)
        self.ny_spinBox.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.ny_spinBox.setFont(font)
        self.ny_spinBox.setMinimum(1)
        self.ny_spinBox.setObjectName(_fromUtf8("ny_spinBox"))
        self.horizontalLayout_5.addWidget(self.ny_spinBox)
        self.horizontalLayout_10.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_8 = QtGui.QHBoxLayout()
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.label_12 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_12.setFont(font)
        self.label_12.setObjectName(_fromUtf8("label_12"))
        self.horizontalLayout_8.addWidget(self.label_12)
        self.Ly_lineEdit = QtGui.QLineEdit(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Ly_lineEdit.sizePolicy().hasHeightForWidth())
        self.Ly_lineEdit.setSizePolicy(sizePolicy)
        self.Ly_lineEdit.setMinimumSize(QtCore.QSize(175, 0))
        self.Ly_lineEdit.setMaximumSize(QtCore.QSize(200, 16777215))
        self.Ly_lineEdit.setReadOnly(True)
        self.Ly_lineEdit.setObjectName(_fromUtf8("Ly_lineEdit"))
        self.horizontalLayout_8.addWidget(self.Ly_lineEdit)
        self.horizontalLayout_10.addLayout(self.horizontalLayout_8)
        spacerItem4 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_12 = QtGui.QHBoxLayout()
        self.horizontalLayout_12.setObjectName(_fromUtf8("horizontalLayout_12"))
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.label_4 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_4.setFont(font)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.horizontalLayout_6.addWidget(self.label_4)
        self.nz_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nz_spinBox.sizePolicy().hasHeightForWidth())
        self.nz_spinBox.setSizePolicy(sizePolicy)
        self.nz_spinBox.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nz_spinBox.setFont(font)
        self.nz_spinBox.setMinimum(1)
        self.nz_spinBox.setMaximum(1000)
        self.nz_spinBox.setProperty("value", 1000)
        self.nz_spinBox.setObjectName(_fromUtf8("nz_spinBox"))
        self.horizontalLayout_6.addWidget(self.nz_spinBox)
        self.horizontalLayout_12.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_9 = QtGui.QHBoxLayout()
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.label_13 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_13.setFont(font)
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.horizontalLayout_9.addWidget(self.label_13)
        self.Lz_doubleSpinBox = QtGui.QDoubleSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Lz_doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.Lz_doubleSpinBox.setSizePolicy(sizePolicy)
        self.Lz_doubleSpinBox.setMinimumSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.Lz_doubleSpinBox.setFont(font)
        self.Lz_doubleSpinBox.setDecimals(4)
        self.Lz_doubleSpinBox.setMaximum(1000.0)
        self.Lz_doubleSpinBox.setObjectName(_fromUtf8("Lz_doubleSpinBox"))
        self.horizontalLayout_9.addWidget(self.Lz_doubleSpinBox)
        self.horizontalLayout_12.addLayout(self.horizontalLayout_9)
        spacerItem5 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.verticalLayout_6.addLayout(self.verticalLayout)
        self.horizontalLayout_38.addLayout(self.verticalLayout_6)
        spacerItem6 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_38.addItem(spacerItem6)
        self.verticalLayout_10.addLayout(self.horizontalLayout_38)
        self.nanogen_tabWidget.addTab(self.nanotube_generator_tabWidget, _fromUtf8(""))
        self.graphene_generator_tabWidget = QtGui.QWidget()
        self.graphene_generator_tabWidget.setObjectName(_fromUtf8("graphene_generator_tabWidget"))
        self.verticalLayout_8 = QtGui.QVBoxLayout(self.graphene_generator_tabWidget)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.horizontalLayout_37 = QtGui.QHBoxLayout()
        self.horizontalLayout_37.setObjectName(_fromUtf8("horizontalLayout_37"))
        self.verticalLayout_7 = QtGui.QVBoxLayout()
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        spacerItem7 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem7)
        self.line_10 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_10.setFrameShape(QtGui.QFrame.HLine)
        self.line_10.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_10.setObjectName(_fromUtf8("line_10"))
        self.verticalLayout_7.addWidget(self.line_10)
        self.horizontalLayout_20 = QtGui.QHBoxLayout()
        self.horizontalLayout_20.setObjectName(_fromUtf8("horizontalLayout_20"))
        self.horizontalLayout_19 = QtGui.QHBoxLayout()
        self.horizontalLayout_19.setObjectName(_fromUtf8("horizontalLayout_19"))
        self.verticalLayout_9 = QtGui.QVBoxLayout()
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.horizontalLayout_16 = QtGui.QHBoxLayout()
        self.horizontalLayout_16.setObjectName(_fromUtf8("horizontalLayout_16"))
        self.label_9 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_9.setFont(font)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.horizontalLayout_16.addWidget(self.label_9)
        self.length_doubleSpinBox = QtGui.QDoubleSpinBox(self.graphene_generator_tabWidget)
        self.length_doubleSpinBox.setMinimumSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.length_doubleSpinBox.setFont(font)
        self.length_doubleSpinBox.setDecimals(4)
        self.length_doubleSpinBox.setMaximum(1000.0)
        self.length_doubleSpinBox.setProperty("value", 10.0)
        self.length_doubleSpinBox.setObjectName(_fromUtf8("length_doubleSpinBox"))
        self.horizontalLayout_16.addWidget(self.length_doubleSpinBox)
        self.verticalLayout_9.addLayout(self.horizontalLayout_16)
        self.horizontalLayout_17 = QtGui.QHBoxLayout()
        self.horizontalLayout_17.setObjectName(_fromUtf8("horizontalLayout_17"))
        self.label_10 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_10.setFont(font)
        self.label_10.setObjectName(_fromUtf8("label_10"))
        self.horizontalLayout_17.addWidget(self.label_10)
        self.width_doubleSpinBox = QtGui.QDoubleSpinBox(self.graphene_generator_tabWidget)
        self.width_doubleSpinBox.setMinimumSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.width_doubleSpinBox.setFont(font)
        self.width_doubleSpinBox.setDecimals(4)
        self.width_doubleSpinBox.setMaximum(1000.0)
        self.width_doubleSpinBox.setProperty("value", 1.0)
        self.width_doubleSpinBox.setObjectName(_fromUtf8("width_doubleSpinBox"))
        self.horizontalLayout_17.addWidget(self.width_doubleSpinBox)
        self.verticalLayout_9.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_19.addLayout(self.verticalLayout_9)
        self.line_5 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_5.setFrameShape(QtGui.QFrame.VLine)
        self.line_5.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_5.setObjectName(_fromUtf8("line_5"))
        self.horizontalLayout_19.addWidget(self.line_5)
        self.horizontalLayout_20.addLayout(self.horizontalLayout_19)
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.label_5 = QtGui.QLabel(self.graphene_generator_tabWidget)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.verticalLayout_3.addWidget(self.label_5)
        self.AC_edge_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        self.AC_edge_radioButton.setChecked(True)
        self.AC_edge_radioButton.setObjectName(_fromUtf8("AC_edge_radioButton"))
        self.Lx_edge_buttonGroup = QtGui.QButtonGroup(NanoGen)
        self.Lx_edge_buttonGroup.setObjectName(_fromUtf8("Lx_edge_buttonGroup"))
        self.Lx_edge_buttonGroup.addButton(self.AC_edge_radioButton)
        self.verticalLayout_3.addWidget(self.AC_edge_radioButton)
        self.ZZ_edge_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        self.ZZ_edge_radioButton.setObjectName(_fromUtf8("ZZ_edge_radioButton"))
        self.Lx_edge_buttonGroup.addButton(self.ZZ_edge_radioButton)
        self.verticalLayout_3.addWidget(self.ZZ_edge_radioButton)
        self.horizontalLayout_20.addLayout(self.verticalLayout_3)
        self.verticalLayout_7.addLayout(self.horizontalLayout_20)
        self.line_4 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_4.setFrameShape(QtGui.QFrame.HLine)
        self.line_4.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_4.setObjectName(_fromUtf8("line_4"))
        self.verticalLayout_7.addWidget(self.line_4)
        self.horizontalLayout_32 = QtGui.QHBoxLayout()
        self.horizontalLayout_32.setObjectName(_fromUtf8("horizontalLayout_32"))
        self.horizontalLayout_33 = QtGui.QHBoxLayout()
        self.horizontalLayout_33.setObjectName(_fromUtf8("horizontalLayout_33"))
        self.horizontalLayout_34 = QtGui.QHBoxLayout()
        self.horizontalLayout_34.setObjectName(_fromUtf8("horizontalLayout_34"))
        self.label_19 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_19.setFont(font)
        self.label_19.setObjectName(_fromUtf8("label_19"))
        self.horizontalLayout_34.addWidget(self.label_19)
        self.graphene_element1_comboBox = QtGui.QComboBox(self.graphene_generator_tabWidget)
        self.graphene_element1_comboBox.setObjectName(_fromUtf8("graphene_element1_comboBox"))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_34.addWidget(self.graphene_element1_comboBox)
        self.horizontalLayout_33.addLayout(self.horizontalLayout_34)
        self.horizontalLayout_35 = QtGui.QHBoxLayout()
        self.horizontalLayout_35.setObjectName(_fromUtf8("horizontalLayout_35"))
        self.label_20 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_20.setFont(font)
        self.label_20.setObjectName(_fromUtf8("label_20"))
        self.horizontalLayout_35.addWidget(self.label_20)
        self.graphene_element2_comboBox = QtGui.QComboBox(self.graphene_generator_tabWidget)
        self.graphene_element2_comboBox.setObjectName(_fromUtf8("graphene_element2_comboBox"))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_35.addWidget(self.graphene_element2_comboBox)
        self.horizontalLayout_33.addLayout(self.horizontalLayout_35)
        self.horizontalLayout_32.addLayout(self.horizontalLayout_33)
        spacerItem8 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_32.addItem(spacerItem8)
        self.verticalLayout_7.addLayout(self.horizontalLayout_32)
        self.horizontalLayout_27 = QtGui.QHBoxLayout()
        self.horizontalLayout_27.setObjectName(_fromUtf8("horizontalLayout_27"))
        self.horizontalLayout_36 = QtGui.QHBoxLayout()
        self.horizontalLayout_36.setObjectName(_fromUtf8("horizontalLayout_36"))
        self.graphene_e1e2_bond_label = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.graphene_e1e2_bond_label.setFont(font)
        self.graphene_e1e2_bond_label.setObjectName(_fromUtf8("graphene_e1e2_bond_label"))
        self.horizontalLayout_36.addWidget(self.graphene_e1e2_bond_label)
        self.graphene_bond_doubleSpinBox = QtGui.QDoubleSpinBox(self.graphene_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.graphene_bond_doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.graphene_bond_doubleSpinBox.setSizePolicy(sizePolicy)
        self.graphene_bond_doubleSpinBox.setMinimumSize(QtCore.QSize(150, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.graphene_bond_doubleSpinBox.setFont(font)
        self.graphene_bond_doubleSpinBox.setInputMethodHints(QtCore.Qt.ImhFormattedNumbersOnly)
        self.graphene_bond_doubleSpinBox.setReadOnly(False)
        self.graphene_bond_doubleSpinBox.setDecimals(3)
        self.graphene_bond_doubleSpinBox.setMinimum(0.5)
        self.graphene_bond_doubleSpinBox.setMaximum(10.0)
        self.graphene_bond_doubleSpinBox.setSingleStep(0.01)
        self.graphene_bond_doubleSpinBox.setProperty("value", 1.42)
        self.graphene_bond_doubleSpinBox.setObjectName(_fromUtf8("graphene_bond_doubleSpinBox"))
        self.horizontalLayout_36.addWidget(self.graphene_bond_doubleSpinBox)
        self.horizontalLayout_27.addLayout(self.horizontalLayout_36)
        spacerItem9 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_27.addItem(spacerItem9)
        self.verticalLayout_7.addLayout(self.horizontalLayout_27)
        self.line_7 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_7.setFrameShape(QtGui.QFrame.HLine)
        self.line_7.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_7.setObjectName(_fromUtf8("line_7"))
        self.verticalLayout_7.addWidget(self.line_7)
        self.horizontalLayout_22 = QtGui.QHBoxLayout()
        self.horizontalLayout_22.setObjectName(_fromUtf8("horizontalLayout_22"))
        self.horizontalLayout_21 = QtGui.QHBoxLayout()
        self.horizontalLayout_21.setObjectName(_fromUtf8("horizontalLayout_21"))
        self.horizontalLayout_18 = QtGui.QHBoxLayout()
        self.horizontalLayout_18.setObjectName(_fromUtf8("horizontalLayout_18"))
        self.label_11 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_11.setFont(font)
        self.label_11.setObjectName(_fromUtf8("label_11"))
        self.horizontalLayout_18.addWidget(self.label_11)
        self.nlayers_spinBox = QtGui.QSpinBox(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nlayers_spinBox.setFont(font)
        self.nlayers_spinBox.setMinimum(1)
        self.nlayers_spinBox.setMaximum(50)
        self.nlayers_spinBox.setObjectName(_fromUtf8("nlayers_spinBox"))
        self.horizontalLayout_18.addWidget(self.nlayers_spinBox)
        self.horizontalLayout_21.addLayout(self.horizontalLayout_18)
        self.line_6 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_6.setFrameShape(QtGui.QFrame.VLine)
        self.line_6.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_6.setObjectName(_fromUtf8("line_6"))
        self.horizontalLayout_21.addWidget(self.line_6)
        self.horizontalLayout_22.addLayout(self.horizontalLayout_21)
        self.verticalLayout_4 = QtGui.QVBoxLayout()
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.label_6 = QtGui.QLabel(self.graphene_generator_tabWidget)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.verticalLayout_4.addWidget(self.label_6)
        self.AA_stacking_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.AA_stacking_radioButton.setFont(font)
        self.AA_stacking_radioButton.setCheckable(True)
        self.AA_stacking_radioButton.setChecked(True)
        self.AA_stacking_radioButton.setObjectName(_fromUtf8("AA_stacking_radioButton"))
        self.stacking_order_buttonGroup = QtGui.QButtonGroup(NanoGen)
        self.stacking_order_buttonGroup.setObjectName(_fromUtf8("stacking_order_buttonGroup"))
        self.stacking_order_buttonGroup.addButton(self.AA_stacking_radioButton)
        self.verticalLayout_4.addWidget(self.AA_stacking_radioButton)
        self.AB_stacking_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.AB_stacking_radioButton.setFont(font)
        self.AB_stacking_radioButton.setCheckable(True)
        self.AB_stacking_radioButton.setObjectName(_fromUtf8("AB_stacking_radioButton"))
        self.stacking_order_buttonGroup.addButton(self.AB_stacking_radioButton)
        self.verticalLayout_4.addWidget(self.AB_stacking_radioButton)
        self.horizontalLayout_22.addLayout(self.verticalLayout_4)
        self.verticalLayout_7.addLayout(self.horizontalLayout_22)
        self.line_9 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_9.setFrameShape(QtGui.QFrame.HLine)
        self.line_9.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_9.setObjectName(_fromUtf8("line_9"))
        self.verticalLayout_7.addWidget(self.line_9)
        spacerItem10 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem10)
        self.horizontalLayout_37.addLayout(self.verticalLayout_7)
        spacerItem11 = QtGui.QSpacerItem(92, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_37.addItem(spacerItem11)
        self.verticalLayout_8.addLayout(self.horizontalLayout_37)
        self.nanogen_tabWidget.addTab(self.graphene_generator_tabWidget, _fromUtf8(""))
        self.fullerene_generators = QtGui.QWidget()
        self.fullerene_generators.setObjectName(_fromUtf8("fullerene_generators"))
        self.verticalLayout_11 = QtGui.QVBoxLayout(self.fullerene_generators)
        self.verticalLayout_11.setObjectName(_fromUtf8("verticalLayout_11"))
        self.listWidget = QtGui.QListWidget(self.fullerene_generators)
        self.listWidget.setObjectName(_fromUtf8("listWidget"))
        self.verticalLayout_11.addWidget(self.listWidget)
        self.nanogen_tabWidget.addTab(self.fullerene_generators, _fromUtf8(""))
        self.verticalLayout_12.addWidget(self.nanogen_tabWidget)
        self.line_3 = QtGui.QFrame(self.centralwidget)
        self.line_3.setFrameShape(QtGui.QFrame.HLine)
        self.line_3.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_3.setObjectName(_fromUtf8("line_3"))
        self.verticalLayout_12.addWidget(self.line_3)
        self.verticalLayout_5 = QtGui.QVBoxLayout()
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.horizontalLayout_25 = QtGui.QHBoxLayout()
        self.horizontalLayout_25.setObjectName(_fromUtf8("horizontalLayout_25"))
        self.label_16 = QtGui.QLabel(self.centralwidget)
        self.label_16.setObjectName(_fromUtf8("label_16"))
        self.horizontalLayout_25.addWidget(self.label_16)
        self.save_directory_lineEdit = QtGui.QLineEdit(self.centralwidget)
        self.save_directory_lineEdit.setObjectName(_fromUtf8("save_directory_lineEdit"))
        self.horizontalLayout_25.addWidget(self.save_directory_lineEdit)
        self.verticalLayout_5.addLayout(self.horizontalLayout_25)
        self.horizontalLayout_15 = QtGui.QHBoxLayout()
        self.horizontalLayout_15.setObjectName(_fromUtf8("horizontalLayout_15"))
        spacerItem12 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem12)
        self.structure_format_comboBox = QtGui.QComboBox(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(18)
        self.structure_format_comboBox.setFont(font)
        self.structure_format_comboBox.setModelColumn(0)
        self.structure_format_comboBox.setObjectName(_fromUtf8("structure_format_comboBox"))
        self.structure_format_comboBox.addItem(_fromUtf8(""))
        self.structure_format_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_15.addWidget(self.structure_format_comboBox)
        self.generate_pushButton = QtGui.QPushButton(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(18)
        self.generate_pushButton.setFont(font)
        self.generate_pushButton.setObjectName(_fromUtf8("generate_pushButton"))
        self.horizontalLayout_15.addWidget(self.generate_pushButton)
        spacerItem13 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem13)
        self.verticalLayout_5.addLayout(self.horizontalLayout_15)
        self.verticalLayout_12.addLayout(self.verticalLayout_5)
        NanoGen.setCentralWidget(self.centralwidget)
        self.nanogen_statusBar = QtGui.QStatusBar(NanoGen)
        self.nanogen_statusBar.setObjectName(_fromUtf8("nanogen_statusBar"))
        NanoGen.setStatusBar(self.nanogen_statusBar)

        self.retranslateUi(NanoGen)
        self.nanogen_tabWidget.setCurrentIndex(0)
        self.nanotube_element1_comboBox.setCurrentIndex(3)
        self.nanotube_element2_comboBox.setCurrentIndex(3)
        self.graphene_element1_comboBox.setCurrentIndex(3)
        self.graphene_element2_comboBox.setCurrentIndex(3)
        self.structure_format_comboBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(NanoGen)

    def retranslateUi(self, NanoGen):
        NanoGen.setWindowTitle(_translate("NanoGen", "NanoGen", None))
        self.nanotube_generator_radioButton.setText(_translate("NanoGen", "Nanotube Generator", None))
        self.capped_nanotube_generator_radioButton.setText(_translate("NanoGen", "Capped Nanotube Generator", None))
        self.mwnt_generator_radioButton.setText(_translate("NanoGen", "Multi-walled Nanotube Generator", None))
        self.unrolled_nanotube_generator_radioButton.setText(_translate("NanoGen", "Unrolled Nanotube Generator", None))
        self.bundle_generator_checkBox.setText(_translate("NanoGen", "Bundle Generator", None))
        self.label_7.setText(_translate("NanoGen", "<html><head/><body><p align=\"right\">n = </p></body></html>", None))
        self.label_14.setText(_translate("NanoGen", "<html><head/><body><p align=\"right\">m = </p></body></html>", None))
        self.label_17.setText(_translate("NanoGen", "Element 1:", None))
        self.nanotube_element1_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.nanotube_element1_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.nanotube_element1_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.nanotube_element1_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.nanotube_element1_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.label_18.setText(_translate("NanoGen", "Element 2:", None))
        self.nanotube_element2_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.nanotube_element2_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.nanotube_element2_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.nanotube_element2_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.nanotube_element2_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.nanotube_e1e2_bond_label.setText(_translate("NanoGen", "<html><head/><body><p>C-C bond =</p></body></html>", None))
        self.nanotube_bond_doubleSpinBox.setSuffix(_translate("NanoGen", " Å", None))
        self.label.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">x </span>=</p></body></html>", None))
        self.label_2.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">x </span>= </p></body></html>", None))
        self.label_3.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">y </span>=</p></body></html>", None))
        self.label_12.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">y </span>= </p></body></html>", None))
        self.label_4.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">z </span>=</p></body></html>", None))
        self.label_13.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">z </span>= </p></body></html>", None))
        self.Lz_doubleSpinBox.setSuffix(_translate("NanoGen", " nm", None))
        self.nanogen_tabWidget.setTabText(self.nanogen_tabWidget.indexOf(self.nanotube_generator_tabWidget), _translate("NanoGen", "Nanotubes", None))
        self.label_9.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">x</span> =</p></body></html>", None))
        self.length_doubleSpinBox.setSuffix(_translate("NanoGen", " nm", None))
        self.label_10.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">y</span> =</p></body></html>", None))
        self.width_doubleSpinBox.setSuffix(_translate("NanoGen", " nm", None))
        self.label_5.setText(_translate("NanoGen", "<html><head/><body><p align=\"center\"><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">x</span> edge</p></body></html>", None))
        self.AC_edge_radioButton.setText(_translate("NanoGen", "armchair", None))
        self.ZZ_edge_radioButton.setText(_translate("NanoGen", "zigzag", None))
        self.label_19.setText(_translate("NanoGen", "Element 1:", None))
        self.graphene_element1_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.graphene_element1_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.graphene_element1_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.graphene_element1_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.graphene_element1_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.label_20.setText(_translate("NanoGen", "Element 2:", None))
        self.graphene_element2_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.graphene_element2_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.graphene_element2_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.graphene_element2_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.graphene_element2_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.graphene_e1e2_bond_label.setText(_translate("NanoGen", "<html><head/><body><p>C-C bond =</p></body></html>", None))
        self.graphene_bond_doubleSpinBox.setSuffix(_translate("NanoGen", " Å", None))
        self.label_11.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">layers</span> =</p></body></html>", None))
        self.label_6.setText(_translate("NanoGen", "<html><head/><body><p>Stacking Order</p></body></html>", None))
        self.AA_stacking_radioButton.setText(_translate("NanoGen", "AA", None))
        self.AB_stacking_radioButton.setText(_translate("NanoGen", "AB", None))
        self.nanogen_tabWidget.setTabText(self.nanogen_tabWidget.indexOf(self.graphene_generator_tabWidget), _translate("NanoGen", "Graphene", None))
        self.nanogen_tabWidget.setTabText(self.nanogen_tabWidget.indexOf(self.fullerene_generators), _translate("NanoGen", "Fullerenes", None))
        self.label_16.setText(_translate("NanoGen", "Save directory:", None))
        self.structure_format_comboBox.setItemText(0, _translate("NanoGen", "LAMMPS data", None))
        self.structure_format_comboBox.setItemText(1, _translate("NanoGen", "xyz", None))
        self.generate_pushButton.setText(_translate("NanoGen", "Generate", None))

