
# THIS FILE IS GENERATED FROM SCIKIT-NANO SETUP.PY
short_version = '0.2.24'
version = '0.2.24'
full_version = '0.2.24'
git_revision = '21db3ce61aa3ba3f8dfd1dd9aa98340bb8772467'
release = True

if not release:
    version = full_version
