# -*- coding: utf-8 -*-
"""
===================================================
Data analysis sub-package (:mod:`sknano.analysis`)
===================================================

.. currentmodule:: sknano.analysis

Contents
========

.. autosummary::
   :toctree: generated/

"""
from __future__ import division, print_function, absolute_import

__docformat__ = 'restructuredtext'

__all__ = [s for s in dir() if not s.startswith('_')]
