# -*- coding: utf-8 -*-
"""
=========================================================
Reference data (:mod:`sknano.tools.refdata._constants`)
=========================================================

.. currentmodule:: sknano.tools.refdata._constants

"""
from __future__ import division, print_function, absolute_import

C_C = CCbond = 1.421  # angstroms
C_H = CHbond = 1.09  # angstroms
dVDW = 3.4  # angstroms

__all__ = ['C_C', 'CCbond', 'C_H', 'CHbond', 'dVDW']
